@@ -354,10 +354,14 @@ oa.design <- function(ID=NULL, nruns=NULL, nfactors=NULL, nlevels=NULL,
            if (repeat.only) orig.no.rp <- paste(orig.no.rp,rep(1:replications,nruns),sep=".")
            else orig.no.rp <- paste(orig.no.rp,rep(1:replications,each=nruns),sep=".")
        }
+    ## row added 27 01 2011 (for proper ordering of design)
+    orig.no.levord <- sort(as.numeric(orig.no),index=TRUE)$ix
       desmat <- model.matrix(~.,data=aus)[,-1,drop=FALSE]
       rownames(aus) <- rownames(desmat) <- 1:nrow(aus)
 
       attr(aus,"desnum") <- desmat
+      ## change 27 Jan 2011: leave orig.no as a factor, but with better-ordered levels
+      orig.no <- factor(orig.no, levels=unique(orig.no[orig.no.levord]))
       attr(aus,"run.order") <- data.frame("run.no.in.std.order"=orig.no,"run.no"=1:nrow(aus),"run.no.std.rp"=orig.no.rp)
       attr(aus,"design.info") <- list(type="oa",
               nruns=nruns,
